function [phase,decomp_mu,decomp_cov,AIC_osc,osc_param] = oscV_decomp(Y,fs,MAX_OSC,MAX_VAR)
%
% Input:
%    Y:             multivariate time series (J times T)
%    MAX_OSC:       maximum number of oscillators (scalar)
%    MAX_VAR:       maximum VAR order (scalar)
%                   (default value = ceil(2*MAX_OSC/J))
%
% Output:
%    phase:         estimated phase of each oscillator (MAX_OSC times T times MAX_OSC)
%                   phi_prop(k,:,K) is the estimated phase of the k-th oscillator in the decomposition into K oscillators
%    decomp_mu:     smoothed coordinate of each oscillator (2*MAX_OSC times T times MAX_OSC)
%                   decomp_mu(2*k-1:2*k,:,K) is the smoothed coordinate of the k-th oscillator in the decomposition into K oscillators
%    decomp_cov:    smoothed covariance of each oscillator (2*MAX_OSC times 2*MAX_OSC times T times MAX_OSC)
%                   decomp_cov(2*k-1:2*k,2*k-1:2*k,:,K) is the smoothed covariance of the k-th oscillator in the decomposition into K oscillators
%    AIC_osc:       AIC of the oscillator model (1 times MAX_OSC)
%                   AIC_osc(K) is the AIC of the oscillator model with K oscillators
%    osc_param:     estimated parameters of the oscillator model (MAX_OSC times (2*J+1)*MAX_OSC+1)
%                   osc_param(K,1:K) is the estimate of a_1,...,a_K in the oscillator model with K oscillators
%                   osc_param(K,K+1:2*K) is the estimate of f_1,...f_K in the oscillator model with K oscillators
%                   osc_param(K,2*K+1:3*K) is the estimate of sigma_1^2,...,sigma_K^2 in the oscillator model with K oscillators
%                   osc_param(K,3*K+1:(2*J+1)*K) is the estimate of c_{21,1},c_{21,2},...,c_{J1,1},c_{J1,2},...,c_{JK,1},c_{JK,2} in the oscillator model with K oscillators
%                   osc_param(K,(2*J+1)*K+1) is the estimate of tau^2 in the oscillator model with K oscillators
%
    options = optimoptions('fminunc','Algorithm','quasi-newton','MaxFunEvals',10000);
	J = size(Y,1);
	T = size(Y,2);
    phase = zeros(MAX_OSC,T,MAX_OSC);
    decomp_mu = zeros(2*MAX_OSC,T,MAX_OSC);
    decomp_cov = zeros(2*MAX_OSC,2*MAX_OSC,T,MAX_OSC);
    AIC_osc = zeros(1,MAX_OSC);
    osc_param = zeros(MAX_OSC,(2*J+1)*MAX_OSC+1);
    MAX_VAR = max(MAX_VAR,ceil(2*MAX_OSC/J));
    VAR_fit;
    tic
	for K=1:MAX_OSC
    	K
	    minAIC = inf;
    	minAIC2 = inf;
	    minK = inf;
    	for ARdeg=ceil(K/J):MAX_VAR
	        [Vtmp,tmp] = polyeig_VAR(VARwithnoise_A(:,1:J*ARdeg,ARdeg));
    	    if J*ARdeg-nnz(imag(tmp))/2 == K && AIC_VARwithnoise(ARdeg) < minAIC
        	    V0 = Vtmp;
            	z0 = tmp;
            	E0 = VARwithnoise_E(:,:,ARdeg);
            	R0 = VARwithnoise_r(ARdeg);
            	minAIC = AIC_VARwithnoise(ARdeg);
    		    optARdeg = ARdeg;
	        end
	        if J*ARdeg-nnz(imag(tmp))/2 > K && (J*ARdeg-nnz(imag(tmp))/2 < minK || (J*ARdeg-nnz(imag(tmp))/2 == minK && AIC_VARwithnoise(ARdeg) < minAIC2))
    	        V1 = Vtmp;
        	    z1 = tmp;
            	E1 = VARwithnoise_E(:,:,ARdeg);
            	R1 = VARwithnoise_r(ARdeg);
            	minAIC2 = AIC_VARwithnoise(ARdeg);
        	    optARdeg2 = ARdeg;
    	        minK = J*ARdeg-nnz(imag(tmp))/2;
	        end
	    end
    	if minAIC == inf
        	if minAIC2 == inf
            	warning('no AR model with %d oscillators',K);
	        end
    	    V0 = V1;
        	z0 = z1;
	        E0 = E1;
    	    R0 = R1;
        	optARdeg = optARdeg2;
	    end
    	[tmp,ARdeg] = min(AIC_VARwithnoise);
	    [Vtmp,tmp] = polyeig_VAR(VARwithnoise_A(:,1:J*ARdeg,ARdeg));
    	if nnz(imag(tmp)>=0) >= K
	        V0 = Vtmp;
    	    z0 = tmp;
        	E0 = VARwithnoise_E(:,:,ARdeg);
        	R0 = VARwithnoise_r(ARdeg);
        	minAIC = AIC_VARwithnoise(ARdeg);
    	    optARdeg = ARdeg;
	    end
	    VV = zeros(J*optARdeg,J*optARdeg);
    	for j=1:J*optARdeg
        	for i=1:optARdeg
            	VV((i-1)*J+1:i*J,j) = z0(j)^(1-i)*V0(:,j);
    	    end
	    end
    	QQ = inv(VV)*[E0 zeros(J,J*(optARdeg-1)); zeros(J*(optARdeg-1),J*optARdeg)]*inv(VV)';
	    [B,I] = sort(diag(real(QQ))./(1-abs(z0).^2),'descend');
    	V0 = V0(:,I);
    	z0 = z0(I);
    	if R0 == 0
    	    R0 = 10^-5;
	    end
	    init_a = zeros(1,K);
    	init_f = zeros(1,K);
    	init_c = zeros(1,2*(J-1)*K);
    	kk = 1;
    	for k=1:K
        	init_a(k) = abs(z0(kk));
    	    init_f(k) = abs(angle(z0(kk)));
	        if isreal(z0(kk)) && z0(kk) < 0
        	    init_f(k) = 4; % for pi
	        end
    	    for ii=1:J-1
        	    init_c(2*(J-1)*(k-1)+2*ii-1) = real(V0(ii+1,kk)/V0(1,kk));
            	if imag(z0(kk)) < 0
                	init_c(2*(J-1)*(k-1)+2*ii) = imag(V0(ii+1,kk)/V0(1,kk));
	            else
    	            init_c(2*(J-1)*(k-1)+2*ii) = -imag(V0(ii+1,kk)/V0(1,kk));
        	    end
	        end
    	    if imag(z0(kk)) == 0
        	    kk = kk+1;
	        else
    	        kk = kk+2;
        	end
	    end
        [B,I] = sort(init_f);
        init_a = init_a(I);
        init_f = init_f(I);
        tmp = init_c;
        for k=1:K
            init_c(2*(J-1)*(k-1)+1:2*(J-1)*k) = tmp(2*(J-1)*(I(k)-1)+1:2*(J-1)*I(k));
        end
    	nf0 = nnz(init_f==0);
    	nf1 = nnz(init_f==4);
	    freq = init_f;
    	freq(2:nf0) = pi*rand(1,nf0-1);
	    freq(K-nf1+2:K) = pi*rand(1,nf1-1);
        init_f(init_f==4) = pi;

        if mod(T,2) == 0
        	freq = [2*pi/T*(0:T/2-1) 4];
	    else
    	    freq = [2*pi/T*(0:(T-1)/2)];
        end
        
        P = zeros(J*length(freq),K);
	    for k=1:K
    	    a = init_a(k);
        	theta = init_f(k);
        	A = (1-2*a^2*cos(theta)^2+a^4*cos(2*theta))/a/(a^2-1)/cos(theta);
        	b = (A-2*a*cos(theta)+sign(cos(theta))*sqrt((A-2*a*cos(theta))^2-4))/2;
        	for j=1:length(freq)
            	P(j,k) = -a*cos(theta)/b*abs(1+b*exp(-1i*freq(j)))^2/abs(1-2*a*cos(theta)*exp(-1i*freq(j))+a^2*exp(-2*1i*freq(j))).^2;
	            for ii=1:J-1
    	            P(ii*length(freq)+j,k) = norm(init_c(2*(J-1)*(k-1)+2*ii-1:2*(J-1)*(k-1)+2*ii))^2*P(j,k);
        	    end
    	    end
	    end
	    [l,bestARdeg] = min(AIC_VARwithnoise);
    	A0 = VARwithnoise_A(:,1:J*bestARdeg,bestARdeg);
	    E0 = VARwithnoise_E(:,:,bestARdeg);
    	p = zeros(J*length(freq),1);
	    for j=1:length(freq)
    	    Phi = eye(J);
        	for k=1:bestARdeg
            	Phi = Phi-A0(:,(k-1)*J+1:k*J)*exp(-1i*k*freq(j));
	        end
    	    tmp = inv(Phi)*E0*inv(Phi)';
        	for ii=1:J
            	p((ii-1)*length(freq)+j) = real(tmp(ii,ii));
    	    end
	    end
	    p = zeros(J*length(freq),1);
    	d1 = zeros(J*length(freq),1);
    	for j=1:length(freq)
        	if freq(j) == 0 || freq(j) == 4
            	d1(j:length(freq):J*length(freq)) = 1;
	        end
    	    for ii=1:J
        	    p((ii-1)*length(freq)+j) = abs(Y(ii,:)*exp(-1i*freq(j)*(0:T-1)'))^2/T;
    	    end
        end
        if cond(P'*P) < 10^6
            init_sigma = (P'*P)\(P'*p);
        else
            init_sigma = expGLMfit(P,p);
        end
        init_sigma(init_sigma<0) = R0;

	    [param,mll] = fminunc(@(param)oscV_ll(Y,param,init_f),[atanh(2*init_a-1) zeros(1,K) log(init_sigma'/R0) init_c],options);
        param(K+1:2*K) = init_f+tanh(param(K+1:2*K))*pi;
    	AIC_osc(K) = 2*mll+2*((2*J+1)*K+1);
    	[mll,osc_r] = oscV_ll(Y,param,init_f);
	    [tmp,I] = sort(abs(param(K+1:2*K)));
    	osc_a = (tanh(param(I))+1)/2;
    	osc_f = abs(param(K+I));
    	osc_sigma = exp(param(2*K+I))*osc_r;
        tmp = zeros(1,2*(J-1)*K);
        for i=1:K
            tmp((i-1)*2*(J-1)+1:i*2*(J-1)) = I(i)-1;
        end
        osc_c = param(3*K+tmp*2*(J-1)+repmat(1:2*(J-1),1,K));
		osc_param(K,1:(2*J+1)*K+1) = [osc_a osc_f*fs/2/pi osc_sigma osc_c osc_r];

	    m = 2*K;
    	x_pred1 = zeros(m,T);
	    x_filt = zeros(m,T);
    	x_smooth = zeros(m,T);
    	V_pred1 = zeros(m,m,T);
    	V_filt = zeros(m,m,T);
    	V_smooth = zeros(m,m,T);
    	F = zeros(m,m);
    	Q = zeros(m,m);
	    for k=1:K
        	F(2*k-1:2*k,2*k-1:2*k) = osc_a(k)*[cos(osc_f(k)) -sin(osc_f(k)); sin(osc_f(k)) cos(osc_f(k))];
    	    Q(2*k-1:2*k,2*k-1:2*k) = osc_sigma(k)*eye(2);
	    end
	    H = zeros(J,m);
    	H(1,1:2:m) = 1;
    	kk = 1;
    	for k=1:K
        	for i=2:J
            	H(i,2*k-1:2*k) = osc_c(kk:kk+1);
        	    kk = kk+2;
    	    end
	    end
	    R = osc_r*eye(J);
    	x_pred1(:,1) = zeros(m,1);
    	for k=1:K
        	V_pred1(2*k-1:2*k,2*k-1:2*k,1) = Q(2*k-1:2*k,2*k-1:2*k)/(1-F(2*k-1,2*k-1)^2-F(2*k-1,2*k)^2);
	    end
    	for t=1:T-1
        	Kg = V_pred1(:,:,t)*H'*inv(H*V_pred1(:,:,t)*H'+R);
    	    x_filt(:,t) = x_pred1(:,t) + Kg*(Y(:,t)-H*x_pred1(:,t));
	        V_filt(:,:,t) = (eye(m)-Kg*H)*V_pred1(:,:,t);
        	x_pred1(:,t+1) = F*x_filt(:,t);
    	    V_pred1(:,:,t+1) = F*V_filt(:,:,t)*F'+Q;
	    end
	    Kg = V_pred1(:,:,T)*H'*inv(H*V_pred1(:,:,T)*H'+R);
    	x_filt(:,T) = x_pred1(:,T) + Kg*(Y(:,T)-H*x_pred1(:,T));
    	V_filt(:,:,T) = (eye(m)-Kg*H)*V_pred1(:,:,T);
    	x_smooth(:,T) = x_filt(:,T);
    	V_smooth(:,:,T) = V_filt(:,:,T);
    	for t=T-1:-1:1
    	    x_smooth(:,t) = x_filt(:,t) + V_filt(:,:,t)*F'*(V_pred1(:,:,t+1)\(x_smooth(:,t+1)-x_pred1(:,t+1)));
	        CC = V_filt(:,:,t)*F'*inv(V_pred1(:,:,t+1));
        	V_smooth(:,:,t) = V_filt(:,:,t) + CC*(V_smooth(:,:,t+1)-V_pred1(:,:,t+1))*CC';
    	end
	    for k=1:K
        	phase(k,:,K) = atan2(x_smooth(2*k,:),x_smooth(2*k-1,:));
    	end
	    decomp_mu(1:m,:,K) = x_smooth;
    	decomp_cov(1:m,1:m,:,K) = V_smooth;
	end
end


